#===- __init__.py - Clang Python Bindings --------------------*- python -*--===#
#
#                     The LLVM Compiler Infrastructure
#
# This file is distributed under the University of Illinois Open Source
# License. See LICENSE.TXT for details.
#
#===------------------------------------------------------------------------===#

r"""
Clang Library Bindings
======================
This package provides access to the Clang compiler and libraries.
The available modules are:
  cindex
    Bindings for the Clang indexing library.
"""

__all__ = ['cindex']
